/// Type alias to represent a register.
type RegisterMem = String;

/// Type alias to represent a label used in place of a const.
pub type ConstId = String;

/// A const can either be a number or a label of some section.
#[derive(Debug)]
pub enum Const {
    CS(ConstId),
    C(u8),
}

/// Operations as they are parsed, before translating them to binary.
/// This type is used internally by the parser and 
/// differs from cpu::Operation.
#[derive(Debug)]
pub enum Operation {
    /// No type.
    NOP,
    HALT,
    /// R type
    ADD(RegisterMem, RegisterMem, RegisterMem),
    SUB(RegisterMem, RegisterMem, RegisterMem),
    AND(RegisterMem, RegisterMem, RegisterMem),
    XOR(RegisterMem, RegisterMem, RegisterMem),
    SLL(RegisterMem, RegisterMem, RegisterMem),
    BEQ(RegisterMem, RegisterMem, RegisterMem),
    BGT(RegisterMem, RegisterMem, RegisterMem),
    LOAD(RegisterMem, RegisterMem, RegisterMem),
    STORE(RegisterMem, RegisterMem, RegisterMem),

    /// I Type
    SLI(RegisterMem, Const),
    ADDI(RegisterMem, Const),
    CALL(RegisterMem, Const),

    /// J Type
    JAL(RegisterMem, RegisterMem, Const),
}
